"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = void 0;
const chalk_1 = __importDefault(require("chalk"));
const globby_1 = __importDefault(require("globby"));
const path_1 = __importDefault(require("path"));
const private_1 = require("../private/index");
const file_1 = require("../../../shared/file/index");
const chokidar_1 = require("chokidar");
const ui_1 = require("../../../shared/ui/index");
/**
 * Watches for changes in any file where an autoindex file resides
 */
function setupWatcherDir(dir, ignored = [], opts) {
    const EVENTS = ["add", "change", "unlink", "link"];
    const h = private_1.watchHandler(dir, opts);
    const watcher = chokidar_1.watch(dir, {
        ignored,
        persistent: true,
        usePolling: true,
        interval: 100,
    });
    watcher.on("ready", () => {
        watcher.on("error", (e) => {
            console.log(chalk_1.default `{red Error occurred watching "${dir}":} ${e.message}\n`);
        });
        for (const evt of EVENTS) {
            watcher.on(evt, h((evt.includes("Dir") ? `${evt}ed directory` : `${evt}ed`).replace("ee", "e")));
        }
    });
    return watcher;
}
/**
 * Watch for changes to autoindex files and add/remove file watchers in response
 */
async function setupAutoIndexWatcher(watched, opts) {
    const log = console.log.bind(console);
    const watcher = chokidar_1.watch("**/index.[jt]s", {
        ignored: "node_modules/*",
        persistent: true,
        usePolling: true,
        interval: 100,
    });
    watcher.on("error", (e) => {
        log(chalk_1.default `{red Error occurred watching for changes to autoindex files:} ${e.message}\n`);
    });
    watcher.on("ready", () => {
        const handlerForAutoIndexFiles = (evt) => {
            return async (filepath) => {
                if (!/(index|private)\.[jt]s/.test(filepath)) {
                    return;
                }
                const watchedDirs = Object.keys(watched);
                const dir = path_1.default.posix.dirname(filepath);
                switch (evt) {
                    case "change":
                        if (watchedDirs.includes(dir) && !private_1.isAutoindexFile(filepath)) {
                            log(chalk_1.default `- index file ${ui_1.highlightFilepath(filepath)} is no longer an {italic autoindex} file`);
                            if (watched[filepath]) {
                                await watched[filepath].close();
                                // watched[filepath] = undefined;
                            }
                            else {
                                log(chalk_1.default `- {red Warn:} an autoindex file was converted to a non autoindexed file but when trying remove the watcher on that directory it appears it doesn't exist. This should not happen.`);
                            }
                        }
                    case "add":
                    case "link":
                        if (!watchedDirs.includes(dir) && private_1.isAutoindexFile(filepath)) {
                            log(`- new autoindex file detected: ${ui_1.highlightFilepath(filepath)}; watcher started`);
                            watched[filepath] = setupWatcherDir(dir, [], opts);
                            private_1.processFiles([filepath], opts);
                        }
                        break;
                    case "unlink":
                        log({ dir, filepath, watched: Object.keys(watched) });
                        if (watchedDirs.includes(dir)) {
                            log(`- the autoindex file ${ui_1.highlightFilepath(filepath)} has been removed`);
                            if (watched[dir]) {
                                await watched[dir].close();
                                // watched[dir] = undefined;
                            }
                            else {
                                log(chalk_1.default `- {red Warn:} an autoindex file was removed but there was no existing watcher on that directory.`);
                            }
                        }
                }
            };
        };
        for (const evt of ["add", "change", "unlink", "link"]) {
            watcher.on(evt, handlerForAutoIndexFiles(evt));
        }
        log(chalk_1.default `{grey - watcher events for autoindex discovery in place}`);
    });
    return watcher;
}
/**
 * Finds all `index.ts` and `index.js` files and looks for the `#autoindex`
 * signature. If found then it _rebuilds_ thes file based on files in
 * the file's current directory
 */
const handler = async ({ opts }) => {
    const monoRepoPackages = file_1.getMonoRepoPackages(process.cwd());
    if (monoRepoPackages && !opts.quiet) {
        console.log(chalk_1.default `{grey - monorepo detected with {yellow ${String(monoRepoPackages.length)}} packages}`);
    }
    const candidateFiles = await globby_1.default(["**/index.ts", "**/index.js", "!**/node_modules"]);
    /** those files known to be autoindex files */
    const autoIndexFiles = candidateFiles.filter((fc) => private_1.isAutoindexFile(fc));
    if (!opts.quiet) {
        if (candidateFiles.length === autoIndexFiles.length) {
            console.log(chalk_1.default `- found {yellow ${String(candidateFiles.length)}} index files, all of which are setup to be autoindexed.\n`);
        }
        else {
            console.log(chalk_1.default `- found {yellow ${String(candidateFiles.length)}} {italic candidate} files, of which {yellow ${String(autoIndexFiles.length)}} have been setup to be autoindexed.\n`);
        }
    }
    await private_1.processFiles(autoIndexFiles, opts);
    if (opts.watch) {
        /**
         * A dictionary of all active watched directories. Keys are the directory path,
         * values are the watcher object.
         */
        const watchedDirs = {};
        setupAutoIndexWatcher(watchedDirs, opts);
        for (const d of autoIndexFiles.map((i) => path_1.default.posix.dirname(i))) {
            watchedDirs[d] = setupWatcherDir(d, [], opts);
        }
        console.log(chalk_1.default `- watching {yellow {bold ${String(autoIndexFiles.length)}}} directories for autoindex changes`);
    }
};
exports.handler = handler;
