"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.serverlessTranspilation = void 0;
const chalk_1 = __importDefault(require("chalk"));
const matcher_1 = __importDefault(require("matcher"));
const npm_1 = require("../../../shared/npm/index");
const serverless_1 = require("../../../shared/serverless/index");
const ast_1 = require("../../../shared/ast/index");
const errors_1 = require("../../../errors/index");
async function filterOutInvalidFunction(fns) {
    const validFns = Object.keys((await serverless_1.getLocalServerlessFunctionsFromServerlessYaml()) || {});
    const results = {
        valid: [],
        invalid: [],
        /** shows fn names which were NOT a direct match but are a soft match */
        explicit: [],
        implicit: [],
    };
    for (const f of fns) {
        if (f.includes("*") || f.includes("!")) {
            // explicit soft match
            results.explicit = [...results.explicit, ...matcher_1.default(validFns, [f])];
        }
        else if (validFns.includes(f)) {
            results.valid.push(f);
        }
        else {
            // implicit soft match
            results.implicit = [...results.explicit, ...matcher_1.default(validFns, [f, `${f}*`])];
            results.invalid.push(f);
        }
    }
    return results;
}
/**
 * Handles any needed transpilation for a **Serverless** project
 */
async function serverlessTranspilation(c) {
    const { argv, opts, config, tooling } = c;
    let fns;
    const validity = await filterOutInvalidFunction(argv);
    fns =
        opts.interactive || argv.length > 0
            ? await serverless_1.askForFunctions(validity.invalid.length > 0
                ? chalk_1.default `Some of the functions you stated were invalid [ {grey ${validity.invalid.join(", ")}} ].\nChoose the functions from the list below:`
                : "Which functions should be transpiled?", [...validity.valid, ...validity.explicit, ...validity.implicit])
            : argv;
    if (opts.force || fns.length > 0) {
        if (npm_1.hasDevDependency("serverless-webpack")) {
            throw new errors_1.DevopsError("You have installed the 'serverless-webpack' plugin which indicates that transpilation will be done by the plugin at \"deploy\" time but you are forcing transpilation at build time.", "do-devops/invalid-transpilation");
        }
        if (fns.length > 0) {
            console.log(chalk_1.default `{grey - transpiling {bold ${String(fns.length)}} handler functions {italic prior} to building {blue serverless.yml}}`);
        }
        fns = fns.length > 0 ? fns : ast_1.getValidServerlessHandlers();
        await tooling({ fns, opts });
        console.log();
    }
    else {
        console.log(chalk_1.default `{grey - {bold Note:} you're using {bold "${config.buildTool}}" for build/bundling which means that by default your TS\nwill not be transpiled to JS during a build. The build process is entirely focused on\nbuilding the "serverless.yml" file. If you want to force transpilation you can do so\nwith the {blue --force} switch.}`);
        console.log(chalk_1.default `\n{grey - {bold Note:} for most people using this config, {blue yarn do watch} will be the most efficient way
to ensure that you always have transpiled code when you {italic deploy}. If you do not then 
the {italic deploy} command will detect this and transpile at deploy-time.}\n`);
    }
}
exports.serverlessTranspilation = serverlessTranspilation;
