"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = void 0;
const chalk_1 = __importDefault(require("chalk"));
const native_dash_1 = require("native-dash");
const table_1 = require("table");
const ui_1 = require("../../../shared/ui/index");
const serverless_1 = require("../../../shared/serverless/index");
const aws_1 = require("../../../shared/aws/index");
const observations_1 = require("../../../shared/observations/index");
const tables_1 = require("./tables");
const file_1 = require("../../../shared/file/index");
const handler = async ({ unknown: argv, opts, observations, }) => {
    const filterBy = argv.length > 0 ? (fn) => fn.includes(argv[0]) : () => true;
    const isServerlessProject = observations.has("serverlessFramework");
    const region = opts.region ? opts.region : await observations_1.determineRegion(opts);
    const stageFilterMsg = opts.stage
        ? chalk_1.default `, filtered down to only those in the {bold ${opts.stage.toUpperCase()}} stage.`
        : "";
    if (!isServerlessProject) {
        if (opts.profile) {
            const filter = opts.stage
                ? (f) => f.FunctionName?.includes(`-${opts.stage}-`)
                : () => true;
            const fns = (await aws_1.getAwsLambdaFunctions(opts)).Functions?.filter(filter);
            if (fns) {
                console.log(chalk_1.default `- AWS functions found using {blue {bold ${opts.profile}}} profile {dim [ ${region} ]}${stageFilterMsg}\n`);
                if (opts.json) {
                    console.log(chalk_1.default `{gray - using {inverse  json } output directly from AWS api instead of a table}`);
                    console.log(fns);
                }
                else {
                    console.log(tables_1.functionsApiTable(fns));
                }
                if (opts.output) {
                    file_1.write(opts.output, fns, { allowOverwrite: true });
                }
            }
            console.log(chalk_1.default `{gray - the AWS CLI provides access to data like this with} {bold aws lambda list-functions --profile ${opts.profile}}`);
        }
        else {
            console.log("- this project does not appear to be a Serverless project!");
            console.log(chalk_1.default `{gray - if you want a list of functions, you can still get this by stating an AWS profile with the "--profile" option}\n`);
        }
        process.exit();
    }
    else if (observations.has("serverlessTs")) {
        if (opts.forceBuild) {
            console.log(`- detected use of the ${chalk_1.default.blue("typescript-microservice")} template; rebuilding functions from config.`);
            await serverless_1.buildLambdaTypescriptProject();
        }
        else {
            console.log(chalk_1.default `- detected use of the {blue typescript-microservice} template; use {bold {blue --forceBuild}} to rebuild prior to listing functions.\n`);
        }
    }
    try {
        const { width } = ui_1.consoleDimensions();
        const fns = (await serverless_1.getServerlessYaml()).functions;
        const tableData = [
            [
                chalk_1.default.bold.yellow("function"),
                chalk_1.default.bold.yellow("events"),
                chalk_1.default.bold.yellow("memory"),
                chalk_1.default.bold.yellow("timeout"),
                chalk_1.default.bold.yellow("description"),
            ],
        ];
        if (fns) {
            for (const key of Object.keys(fns).filter(filterBy)) {
                const events = fns[key].events || [];
                tableData.push([
                    key,
                    events.map((i) => Object.keys(i)).join(", "),
                    String(fns[key].memorySize || chalk_1.default.grey("1024")),
                    String(fns[key].timeout || chalk_1.default.grey("3")),
                    fns[key].description || "",
                ]);
            }
        }
        let tableConfig = {
            columns: {
                0: { width: 30, alignment: "left" },
                1: { width: 16, alignment: "left" },
                2: { width: 7, alignment: "center" },
                3: { width: 10, alignment: "center" },
                4: { width: 46, alignment: "left" },
            },
        };
        let output = table_1.table(tableData, tableConfig);
        if (width < 70) {
            // TODO: come back and use a consistent means of omitting columns
            tableConfig = { columns: native_dash_1.omit(tableConfig.columns, "2", "3", "4") };
            output = table_1.table(tableData.map((i) => i.slice(0, 2), tableConfig));
        }
        else if (width < 80) {
            delete tableConfig.columns["3"];
            delete tableConfig.columns["4"];
            output = table_1.table(tableData.map((i) => i.slice(0, 3), tableConfig));
        }
        else if (width < 125) {
            delete tableConfig.columns["4"];
            output = table_1.table(tableData.map((i) => i.slice(0, 4), tableConfig));
        }
        console.log(output);
    }
    catch (error) {
        console.log(`- Error finding functions: ${error.message}\n`);
        process.exit();
    }
};
exports.handler = handler;
