"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.thisRepo = void 0;
const async_shelljs_1 = require("async-shelljs");
const chalk_1 = __importDefault(require("chalk"));
const path_1 = __importDefault(require("path"));
const date_fns_1 = require("date-fns");
const table_1 = require("table");
const shared_1 = require("../../../shared/index");
const file_1 = require("../../../shared/file/index");
const git_1 = require("../../../shared/git/index");
const interactive_1 = require("../../../shared/interactive/index");
const npm_1 = require("../../../shared/npm/index");
const ui_1 = require("../../../shared/ui/index");
/**
 * if user adds packages to the `pp info [p1] [p2]` this function will respond
 */
async function thisRepo(opts, observations) {
    // const config = await getConfig();
    let npm;
    try {
        npm = await npm_1.getPackageInfo();
    }
    catch {
        // appears NOT to be a NPM package
    }
    const pkg = npm_1.getPackageJson();
    const linkedRepos = file_1.symlinks(file_1.currentDirectory("node_modules")).filter((s) => !s.linkTo.startsWith(".pnpm/"));
    const pkgManagerConflict = observations.has("packageManagerConflict")
        ? chalk_1.default `- {yellow {bold ${"\u2622\uFE0F" /* hazzard */}}} there are conflicts in lock files from more than one package manager!`
        : false;
    if (pkgManagerConflict) {
        console.log(pkgManagerConflict);
        const cont = await interactive_1.askConfirmQuestion("Resolve this conflict?");
        if (cont) {
            await interactive_1.resolvePackageManagerConflict(observations);
        }
    }
    observations = shared_1.getObservations();
    const linked = linkedRepos.length > 0
        ? [
            "Linked\nRepos",
            linkedRepos
                .map((r) => chalk_1.default `{bold ${r.file} -} {dim ${path_1.default.posix.resolve(r.linkTo)}}`)
                .join("\n"),
        ]
        : undefined;
    const pkgExports = [
        { name: "commonjs", value: pkg.main },
        { name: "es-module", value: pkg.module },
        { name: "typings", value: pkg.typings || pkg.types },
    ].filter((i) => i.value);
    const exportsRow = pkgExports.length > 0
        ? ["Exports", pkgExports.map((i) => chalk_1.default `{bold ${i.name}} - {dim ${i.value}}`).join("\n")]
        : ["", ""];
    const priorVersions = npm
        ? npm.versions
            .filter((i) => i !== npm?.version)
            .slice(0, 5)
            .join(", ")
        : "";
    const gitLastCommit = await git_1.getGitLastCommit();
    const branch = await git_1.getCurrentGitBranch();
    const localFilesChanged = (await async_shelljs_1.asyncExec("git diff --name-only", {
        silent: true,
    })).split("\n").length;
    const dateFormat = "ddd dd MMM yyyy";
    /**
     * NPM Info based on verbose flag
     */
    const npmInfo = [
        [
            true,
            npm
                ? chalk_1.default `This repo was first published on {green ${date_fns_1.format(date_fns_1.parseISO(npm.time.created), dateFormat)}} and last modified on {green ${date_fns_1.format(date_fns_1.parseISO(npm.time.modified), dateFormat)}}.\n\n`
                : "",
        ],
        [
            false,
            npm
                ? chalk_1.default `The latest published version is ${chalk_1.default.bold.green(npm.version)} [ ${date_fns_1.format(date_fns_1.parseISO(npm.time[npm.version]), dateFormat)} ].\nLocally in package.json, version is ${chalk_1.default.bold.green(pkg.version)}.`
                : chalk_1.default `Locally in {italic package.json}, the version is ${chalk_1.default.bold.green(pkg.version)} but this is {italic not} an npm package.`,
        ],
        [true, chalk_1.default `\n\nPrior versions include: {italic ${priorVersions}}`],
        [
            true,
            npm && npm.author
                ? chalk_1.default `\n\nThe author of the repo is {green {bold ${typeof npm.author === "string" ? npm.author : npm.author.name}${typeof npm.author === "object" && npm.author.email ? ` <${npm.author.email}>` : ""}}}`
                : "",
        ],
    ];
    const depsSummary = `This repo has ${ui_1.green(Object.keys(pkg?.dependencies || {}).length)} dependencies${npm
        ? chalk_1.default `, with a total of ${ui_1.green(npm.dist.fileCount)} files\nand a unpacked size of ${ui_1.green(npm.dist.unpackedSize / 1000, chalk_1.default ` {italic kb}`)}.`
        : "."}`;
    const depDetails = `${depsSummary}\n\nThe dependencies are:\n - ${ui_1.dim(Object.keys(pkg?.dependencies || {}).join("\n - "))}`;
    console.log(chalk_1.default `Info on package {green {bold ${pkg.name}}}\n`);
    const data = [
        [
            chalk_1.default.bold("Desc"),
            pkg.description ? pkg.description : chalk_1.default.bold.italic("no description provided!"),
        ],
        ...exportsRow,
        [
            "NPM",
            npmInfo
                .filter((i) => opts.verbose || !i[0])
                .map((i) => i[1])
                .join(""),
        ],
        [chalk_1.default.bold("Deps"), opts.verbose === true ? depDetails : depsSummary],
        [
            "Repo ",
            pkg.repository && typeof pkg.repository === "object"
                ? git_1.convertGitUrlToHttp(pkg.repository.url)
                : pkg.repository
                    ? git_1.convertGitUrlToHttp(pkg.repository)
                    : chalk_1.default.red(`The repository is ${chalk_1.default.bold("not")} stated!`),
        ],
        ["Scripts", Object.keys(pkg?.scripts || {}).join(", ")],
        [
            "GIT",
            `Latest commit ${ui_1.green(gitLastCommit)} ${chalk_1.default.bold.italic("@ " + branch)}\n${ui_1.green(String(localFilesChanged))} files changed locally`,
        ],
        ["Tags ", pkg.keywords ? pkg.keywords.join(", ") : chalk_1.default.italic.dim("none")],
    ].filter((i) => Array.isArray(i) && i[0]);
    if (linked) {
        data.push(linked);
    }
    const tblConfig = {
        columns: [{ alignment: "left" }, { width: 69, wrapWord: true }],
    };
    console.log(table_1.table(data, tblConfig));
}
exports.thisRepo = thisRepo;
