"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute = void 0;
const chalk_1 = __importDefault(require("chalk"));
const aws_1 = require("../../../../shared/aws/index");
const native_dash_1 = require("native-dash");
const aws_ssm_1 = require("aws-ssm");
const date_fns_1 = require("date-fns");
const table_1 = require("table");
const observations_1 = require("../../../../shared/observations/index");
const errors_1 = require("../../../../errors/index");
const ui_1 = require("../../../../shared/ui/index");
const execute = async ({ opts, unknown }) => {
    const profile = await observations_1.determineProfile({ ...opts, interactive: true });
    if (!profile) {
        console.log(chalk_1.default `- Couldn't determine the AWS Profile; try setting it manually with {inverse  --profile }.`);
        console.log(chalk_1.default `- alternatively use the {inverse --interactive } option to have the CLI interactively let you select`);
        process.exit();
    }
    const profileInfo = await aws_1.getAwsProfile(profile);
    const region = opts.region || profileInfo.region || (await observations_1.determineRegion({ ...opts, interactive: true }));
    const secrets = unknown;
    const nonStandardPath = opts.nonStandardPath || false;
    const { width } = ui_1.consoleDimensions();
    if (!region) {
        throw new errors_1.DevopsError(`Getting SSM secrets requires an ${chalk_1.default.bold("AWS Region")} and none could be deduced. You can explicitly state this by adding "--region XYZ" to the command.`);
    }
    if (!profile) {
        throw new errors_1.DevopsError(`Getting SSM secrets requires an ${chalk_1.default.bold("AWS Profile")} and none could be deduced. You can explicitly state this by adding "--profile XYZ" to the command.`);
    }
    if (!opts.quiet) {
        console.log(`- Getting SSM details for: ${chalk_1.default.italic.grey.bold(secrets.join(", "))}\n`);
    }
    const tableConfig = {
        columns: {
            0: { width: 30, alignment: "left" },
            1: { width: width > 125 ? 60 : width > 100 ? 40 : 35 },
            2: { width: 8, alignment: "center" },
            3: { width: 16, alignment: "center" },
        },
    };
    const ssm = new aws_ssm_1.SSM({ profile, region });
    for await (const secret of secrets) {
        const tableData = [
            [
                chalk_1.default.yellow.bold("Path"),
                chalk_1.default.yellow.bold("ARN"),
                chalk_1.default.yellow.bold("Version"),
                chalk_1.default.yellow.bold("LastUpdated"),
            ],
        ];
        const data = await ssm.get(secret, { decrypt: true, nonStandardPath });
        tableData.push([
            data.path,
            data.arn,
            String(data.version),
            date_fns_1.format(data.lastUpdated, "dd MMM, yyyy"),
        ]);
        const value = opts.base64 ? native_dash_1.fromBase64(String(data.value)) : String(data.value);
        if (!opts.quiet) {
            console.log(table_1.table(tableData, tableConfig));
            console.log(chalk_1.default.yellow.bold("VALUE:\n"));
            console.log(value);
            console.log();
        }
        else {
            console.log(value);
        }
    }
};
exports.execute = execute;
