"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.write = void 0;
const fs_1 = require("fs");
const errors_1 = require("../../../errors/index");
const existance_1 = require("../existance/index");
const helpers_1 = require("../helpers/index");
/**
 * **write**
 *
 * Writes a file to the filesystem.
 *
 * **Errors:**
 * - `do-devops/file-exists`
 * - `do-devops/file-write-error`
 *
 * **Note:** _if the filename is prefixed with a_ `.`, `~`, _or_ `/` _then it will be considered a
 * full file path but in other cases it will always be offset from the current working
 * directory._
 */
function write(filename, data, options = {}) {
    try {
        // get data into a string form
        const content = typeof data !== "string"
            ? options.pretty
                ? JSON.stringify(data, null, 2)
                : JSON.stringify(data)
            : data;
        filename = helpers_1.interpolateFilePath(filename);
        // avoid collisions if offset avoidence is enabled
        let offset;
        while (options.offsetIfExists && existance_1.fileExists(filename)) {
            const before = new RegExp(`-${offset}.(.*)$`);
            filename = offset ? filename.replace(before, ".$1") : filename;
            offset = !offset ? 1 : offset++;
            // const after = new RegExp(`-${offset}$`);
            const parts = filename.split(".");
            filename = parts.slice(0, -1).join(".") + `-${offset}.` + parts.slice(-1);
        }
        if (!options.offsetIfExists && !options.allowOverwrite && existance_1.fileExists(filename)) {
            throw new errors_1.DevopsError(`The file "${filename}" already exists and the {italic overwrite} flag was not set. Write was not allowed.`, "do-devops/file-exists");
        }
        fs_1.writeFileSync(filename, content, {
            encoding: "utf-8",
        });
        return { filename, data };
    }
    catch (error) {
        if (errors_1.isClassification(error, "do-devops/file-exists")) {
            throw error;
        }
        throw new errors_1.DevopsError(`Problem writing file "${filename}": ${error.message}`, "do-devops/file-write-error");
    }
}
exports.write = write;
