"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.determinePackageManager = void 0;
const errors_1 = require("../../errors/index");
const interactive_1 = require("../interactive/index");
const core_1 = require("../core/index");
const npm_1 = require("../npm/index");
const chalk_1 = __importDefault(require("chalk"));
/**
 * Based on all observations made at startup, this function will try to
 * determine which package manager is being used in the current repo.
 *
 * If it can not be determined, you may optionally switch to interactive mode
 * to get this information but otherwise it will return false.
 *
 * Note: if there's no `package.json` in current directory it will throw an error.
 */
async function determinePackageManager(observations, interactive = false) {
    if (!observations.has("packageJson")) {
        throw new errors_1.DevopsError(`Can not determine the default package manager in a directory that has no package.json file!`, "not-ready/missing-package-json");
    }
    if (observations.has("packageManagerConflict")) {
        console.log(`- ${"\u26A0\uFE0F" /* warn */}} there are indications of {italic more} than one package manager being used!`);
        const pkgManager = await interactive_1.askListQuestion("Which package manager do you expect to use in this repo?", ["npm", "pnpm", "yarn"], observations.has("pnpm") ? "pnpm" : observations.has("yarn") ? "yarn" : "npm");
        await core_1.saveToProjectConfig("general", { pkgManager });
        const removed = await npm_1.removeOtherLockFiles(pkgManager);
        if (removed.length > 0) {
            console.log(chalk_1.default `- removed `);
        }
    }
    if (observations.has("yarn")) {
        return "yarn";
    }
    if (observations.has("npm")) {
        return "npm";
    }
    if (observations.has("pnpm")) {
        return "pnpm";
    }
    if (interactive) {
        const answer = await interactive_1.askListQuestion("We couldn't determine your default package manager, please choose from the list.", ["npm", "pnpm", "yarn", "none"], "npm");
        return answer === "none" ? false : answer;
    }
    return false;
}
exports.determinePackageManager = determinePackageManager;
