"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createInlineExports = void 0;
/* eslint-disable quotes */
const chalk_1 = __importDefault(require("chalk"));
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const common_types_1 = require("common-types");
const file_1 = require("../../file/index");
const ast_1 = require("../../ast/index");
function objectPrint(obj) {
    const contents = [];
    for (const key of Object.keys(obj)) {
        let value = obj[key];
        if (typeof value === "string") {
            value = `"${value.replace(/"/g, '\\"')}"`;
        }
        if (typeof value === "object") {
            value = JSON.stringify(value);
        }
        contents.push(`  ${key}: ${value}`);
        contents.join(",\n\t");
        continue;
    }
    return contents;
}
function warnAboutMissingTyping(config) {
    const incorrectOrMissingTyping = config.filter((i) => i.interface !== "IHandlerConfig");
    if (incorrectOrMissingTyping.length > 0) {
        console.log(chalk_1.default `- there were ${String(incorrectOrMissingTyping.length)} handler functions who defined a {italic config} but did not type it as {bold IHandlerConfig}`);
        console.log(chalk_1.default `{grey - the function configs needing attention are: {italic ${incorrectOrMissingTyping
            .map((i) => common_types_1.isServerlessFunctionHandler(i.config) ? i.config.handler : i.config.image)
            .join(", ")}}}`);
    }
}
/**
 * Writes the serverless configuration file which contains
 * all the _inline_ function definitions found under `src/handlers`.
 *
 * **Note:** if the build tool is _webpack_ and the `serverless-webpack`
 * plugin is _not_ installed then it the inline functions will instead
 * be pointed to the transpiled location in the `.webpack` directory with
 * an `package: { artifact: fn.zip }`
 */
async function createInlineExports(handlers) {
    // const bespokeWebpack =
    //   (await getConfig()).build.buildTool === "webpack" &&
    //   !hasDevDependency("serverless-webpack");
    const header = 'import { IServerlessFunction } from "common-types";\n';
    const body = [];
    const config = [];
    for (const handler of handlers) {
        // const comments = findHandlerComments(handler);
        const handlerConfig = ast_1.findHandlerConfig(handler.source, /** bespokeWebpack*/ false);
        if (handlerConfig && handlerConfig.interface) {
            config.push(handlerConfig);
        }
        else {
            console.log(chalk_1.default `- ${"\uD83D\uDCA9" /* poop */} the {red ${file_1.toRelativePath(handler.source)}} file will be ignored as a handler as it has no CONFIG section defined. This is probably a mistake!`);
        }
    }
    const exportSymbols = [];
    warnAboutMissingTyping(config);
    for (const handler of config) {
        if (common_types_1.isServerlessFunctionHandler(handler.config)) {
            const fnName = (handler?.config?.handler.split("/").pop() || "").replace(/\.[^.]+$/, "");
            exportSymbols.push(fnName);
            const symbol = `const ${fnName}: IServerlessFunction = { 
  ${objectPrint(handler.config)}
  }
  `;
            body.push(symbol);
        }
        else {
            console.warn(`[${handler.config.image}]: the serverless function passed into createInlineExports appears to define an "image" rather than a "handler". This should be investigated!`);
        }
    }
    const file = `
${header}
${body.join("\n")}

export default {
  ${exportSymbols.join(",\n\t")}
}`;
    fs_1.writeFileSync(path_1.default.join(process.env.PWD || "", "serverless-config/functions/inline.ts"), file, {
        encoding: "utf-8",
    });
}
exports.createInlineExports = createInlineExports;
