"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeServerlessFunctionExports = exports.createFunctionDictionary = exports.getFilenameWithoutExtension = exports.reduceToRelativePath = void 0;
const path_1 = __importDefault(require("path"));
const serverless_1 = require("../index");
const index_1 = require("./index");
const fs_1 = require("fs");
/**
 * **reduceToRelativePath**
 *
 * Reduces the file path to just the relative path _beyond_ the passed in `root` path
 */
function reduceToRelativePath(root, fullyQualifiedPath) {
    return fullyQualifiedPath.replace(root, "");
}
exports.reduceToRelativePath = reduceToRelativePath;
/**
 * **getFilenameWithoutExtension**
 *
 * Given a path, filename, and extension (including `.def.ts` as an extension type);
 * this function will return just the filename component.
 */
function getFilenameWithoutExtension(filePath) {
    return (filePath.split("/").pop() || "").split(".")[0];
}
exports.getFilenameWithoutExtension = getFilenameWithoutExtension;
async function createFunctionDictionary(rootPath) {
    const root = rootPath || process.env.PWD || "";
    const fns = serverless_1.findInlineFunctionDefnFiles(rootPath || path_1.default.join(process.env.PWD || "'", "/src"));
    const serverlessNameLookup = index_1.getNamespacedLookup(fns, root);
    const { valid } = await index_1.validateExports(fns);
    return fns.map((filePath) => {
        return {
            filePath,
            fileDir: index_1.getFilePath(filePath),
            relativePath: reduceToRelativePath(root, filePath),
            validHandlerDefinition: valid.includes(filePath),
            configFilename: getFilenameWithoutExtension(filePath) + ".def.ts",
            fnFilename: getFilenameWithoutExtension(filePath) + ".ts",
            serverlessFn: serverlessNameLookup[filePath],
        };
    });
}
exports.createFunctionDictionary = createFunctionDictionary;
/**
 * **writeServerlessFunctionExports**
 *
 * writes all serverless function definition/configurations to the
 * `serverless-config/functions.ts` file. This file will not only
 * export all function definitions but will also provide a typescript
 * **type** called `IDefinedServerlessFunction` which will be a set of
 * string names which are defined in the given repo
 *
 * @param basePath you may alternatively state a base file path to use
 * when looking for function definition files (aka, files named `*.defn.ts`)
 * @param output rather than exporting to the file `serverless-config/functions.ts` you
 * may state an alternative
 */
async function writeServerlessFunctionExports(basePath, output) {
    const outputFilename = output || path_1.default.join(process.env.PWD || "", "/serverless-config/functions.ts");
    // const functionDefns = findInlineFunctionDefnFiles(basePath).map((p) =>
    //   reduceToRelativePath(root, p)
    // );
    const dict = await createFunctionDictionary(basePath);
    let template = "##imports##\n\n##exports##\n\n##interface##";
    template = template.replace("##imports##", dict
        .map((i) => `${i.validHandlerDefinition
        ? `import ${i.serverlessFn} from '.${i.relativePath.replace(".ts", "")}';`
        : `// invalid handler definition for "${i.serverlessFn}"; please check handler definition and then rebuild `}`)
        .join("\n"));
    template = template.replace("##exports##", dict
        .filter((i) => i.validHandlerDefinition)
        .map((i) => `export { ${i.serverlessFn} };`)
        .join("\n"));
    template = template.replace("##interface##", "export type IDefinedServerlessFunction = " +
        dict
            .filter((i) => i.validHandlerDefinition)
            .map((i) => `'${i.serverlessFn}'`)
            .join(" | "));
    fs_1.writeFileSync(outputFilename, "/**\n * DO NOT CHANGE THIS FILE\n * (this file is automatically created)\n **/\n\n" +
        template);
}
exports.writeServerlessFunctionExports = writeServerlessFunctionExports;
