"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toTable = exports.isColumnTuple = exports.isAdvancedColumn = void 0;
const chalk_1 = __importDefault(require("chalk"));
const table_1 = require("table");
const consoleDimensions_1 = require("./consoleDimensions");
function isAdvancedColumn(col) {
    return col !== null && typeof col === "object";
}
exports.isAdvancedColumn = isAdvancedColumn;
function isColumnTuple(col) {
    return Array.isArray(col);
}
exports.isColumnTuple = isColumnTuple;
function headerCol(name) {
    return chalk_1.default.bold.blue(name);
}
/**
 * **toTable**
 *
 * Utility function to take an array of values -- in dictionary form --
 * and return them back in a form which can be brought into the `table`
 * library as data.
 */
function toTable(data, ...columns) {
    const ui = consoleDimensions_1.consoleDimensions();
    const hiddenColumns = [];
    const headerRow = [];
    const colConfig = [];
    // Show Headers First
    for (const col of columns) {
        if (isAdvancedColumn(col)) {
            if (col.minWidth === undefined || col.minWidth < ui.width) {
                headerRow.push(headerCol(col.name || col.col));
                if (col.format) {
                    colConfig.push(col.format);
                }
            }
            else {
                hiddenColumns.push(col.name || col.col);
            }
        }
        else {
            headerRow.push(headerCol(Array.isArray(col) ? col[0] : col));
            colConfig.push({});
        }
    }
    const rows = data.map((rowData) => {
        const row = [];
        for (const col of columns) {
            if (isColumnTuple(col)) {
                const [prop, fn] = col;
                row.push(fn(rowData[prop]));
            }
            else if (isAdvancedColumn(col)) {
                if (col.minWidth === undefined || col.minWidth < ui.width) {
                    row.push(col.formula ? col.formula(rowData[col.col]) : rowData[col.col]);
                }
            }
            else {
                row.push(rowData[col]);
            }
        }
        return row;
    });
    const hiddenColMessage = hiddenColumns.length > 0
        ? chalk_1.default `\n\n{gray - {bold Note:} some columns hidden due to available width [{dim ${ui.width}}]: {italic ${hiddenColumns.join(", ")}}}`
        : "";
    // Iterate through data
    const tableData = [headerRow, ...rows];
    try {
        return table_1.table(tableData, { columns: colConfig }) + hiddenColMessage;
    }
    catch (error) {
        console.log(`- ${"\uD83D\uDCA9" /* poop */} problems building table: ${error.message}`);
        if (error.message === "Table must have a consistent number of cells.") {
            console.log("     - table row lengths are:", tableData.map((i) => i.length));
            console.log();
        }
        process.exit();
    }
}
exports.toTable = toTable;
